@@ -1529,7 +1529,7 @@ Will use HTML5 for this SASsession.""")
             if   dts[col] == 'N' and var == 'nan':
                var = '.'
             elif dts[col] == 'C':
-               if var == 'nan':
+               if var == 'nan' or len(var) == 0:
                   var = ' '
                else:
                   var = var.replace(colsep, colrep)
@@ -1629,16 +1629,16 @@ Will use HTML5 for this SASsession.""")
       vartype = l2[2].split("\n", nvars)
       del vartype[nvars]
 
-      topts             = dict(dsopts)
-      topts['obs']      = 0
-      topts['firstobs'] = ''
-      
+      topts = dict(dsopts)
+      topts.pop('firstobs', None)
+      topts.pop('obs', None)
+   
       code  = "data work._n_u_l_l_;output;run;\n"
       code += "data _null_; file STDERR; set work._n_u_l_l_ "+tabname+self._sb._dsopts(topts)+";put 'FMT_CATS=';\n"
 
       for i in range(nvars):
          code += "_tom = vformatn('"+varlist[i]+"'n);put _tom;\n"
-      code += "run;\nproc delete data=work._n_u_l_l_;run;"
+      code += "stop;\nrun;\nproc delete data=work._n_u_l_l_;run;"
       
       ll = self.submit(code, "text")
       
@@ -1904,16 +1904,16 @@ Will use HTML5 for this SASsession.""")
       vartype = l2[2].split("\n", nvars)
       del vartype[nvars]
 
-      topts             = dict(dsopts)
-      topts['obs']      = 0
-      topts['firstobs'] = ''
-
+      topts = dict(dsopts)
+      topts.pop('firstobs', None)
+      topts.pop('obs', None)
+   
       code  = "data work._n_u_l_l_;output;run;\n"
       code += "data _null_; file STDERR; set work._n_u_l_l_ "+tabname+self._sb._dsopts(topts)+";put 'FMT_CATS=';\n"
 
       for i in range(nvars):
          code += "_tom = vformatn('"+varlist[i]+"'n);put _tom;\n"
-      code += "run;\nproc delete data=work._n_u_l_l_;run;"
+      code += "stop;\nrun;\nproc delete data=work._n_u_l_l_;run;"
 
       ll = self.submit(code, "text")
 
@@ -2123,16 +2123,16 @@ Will use HTML5 for this SASsession.""")
       vartype = l2[2].split("\n", nvars)
       del vartype[nvars]
 
-      topts             = dict(dsopts)
-      topts['obs']      = 0
-      topts['firstobs'] = ''
-
+      topts = dict(dsopts)
+      topts.pop('firstobs', None)
+      topts.pop('obs', None)
+   
       code  = "data work._n_u_l_l_;output;run;\n"
       code += "data _null_; file STDERR; set work._n_u_l_l_ "+tabname+self._sb._dsopts(topts)+";put 'FMT_CATS=';\n"
 
       for i in range(nvars):
          code += "_tom = vformatn('"+varlist[i]+"'n);put _tom;\n"
-      code += "run;\nproc delete data=work._n_u_l_l_;run;"
+      code += "stop;\nrun;\nproc delete data=work._n_u_l_l_;run;"
 
       ll = self.submit(code, "text")
 
